#!/usr/bin/env python

import gtk
import gobject # for threads_init()
import basicscene
import math
import time
import ssrsocket
import packetParser
import sys


# default settings,
HOSTNAME = "localhost"
PORT = 4711

#commandline... 
if len(sys.argv) == 2:
	splitted= sys.argv[1].split(":")
	if len(splitted) == 1:
		HOSTNAME = splitted[0]
	if len(splitted) == 2:
		HOSTNAME = splitted[0]
		PORT     = int( splitted[1] )

#gtk.gdk.threads_init()
gobject.threads_init() # gtk.gdk.threads_init() didn't work in MS Windows
quit = False

class TransportActionGroup( gtk.ActionGroup ):
	def __init__( self, ssrsock ):
		gtk.ActionGroup.__init__( self, "transport_group" )
		self.ss = ssrsock

		self.transport_actions=[
				("play", gtk.STOCK_MEDIA_PLAY, "Play", "<control>s", None, self.play_cb),
				("pause", gtk.STOCK_MEDIA_PAUSE, "Pause", "<control>p", None, self.pause_cb),
				("rewind", gtk.STOCK_MEDIA_PREVIOUS, "Rewind", "<control>r", None, self.rewind_cb)
				]
				
		self.add_actions( self.transport_actions )

	def play_cb( self, b ):
		cmd = "<request><transport action='start'/></request>\0"
		self.ss.push( cmd )

	def pause_cb( self, b ):
		cmd = "<request><transport action='stop'/></request>\0"
		self.ss.push( cmd )

	def rewind_cb( self, b ):
		cmd = "<request><transport time='0'/></request>\0"
		self.ss.push( cmd )


class SSRDraw( gtk.DrawingArea ):
	def __init__( self ):
		gtk.DrawingArea.__init__( self )
		self.modify_bg( gtk.STATE_NORMAL, gtk.gdk.Color() )
		self.connect( "expose-event", self.onExpose )
		self.connect( "button-press-event", self.onButtonPress )
		self.connect( "button-release-event", self.onButtonRelease )
		self.connect("motion_notify_event", self.motion_notify_event)
		self.scene = basicscene.BasicScene()
		self.set_events(  gtk.gdk.EXPOSURE_MASK
				| gtk.gdk.BUTTON_RELEASE_MASK
				| gtk.gdk.BUTTON_PRESS_MASK
				| gtk.gdk.POINTER_MOTION_MASK
				| gtk.gdk.POINTER_MOTION_HINT_MASK )
		self.selected_sources = []
		self.dragging = False
		self.draggingReference = False
		 

	def onExpose( self, widget, event ):
		cr = widget.window.cairo_create()
		alloc = widget.get_allocation()
		cr.scale( alloc.width, alloc.height )
		cr.translate( 0.5, 0.5 )
		cr.scale( 0.2, -0.2 )
		cr.set_source_rgb( 0.7, 0.7, 0.7 )

		self.drawLoudspeakers( cr )

		for i in self.scene.sources.keys():
			self.drawSource( cr, self.scene.sources[i] )

	def onButtonPress( self, widget, event ):
		if( event.button == 1 ):
			alloc = widget.get_allocation()
			x = (float(event.x)/alloc.width-0.5)*5
			y = (float(event.y)/alloc.height-0.5)*-5
			print x,y
			self.selected_sources = []

			if (abs(x-self.scene.reference.x)<0.2) and (abs(y-self.scene.reference.y)<0.2):
				self.dragging = True
				self.draggingReference = True
				return True

			for id,src in self.scene.sources.items():
				if (abs(x-src.x)<0.2) and (abs(y-src.y)<0.2):
					print src.name
					self.selected_sources=[id]
					self.dragging = True

		return True
	
	def onButtonRelease( self, widget, event ):
		self.dragging = False
		self.draggingReference = False
		return True


	def motion_notify_event(self, widget, event):
		if event.is_hint:
			x, y, state = event.window.get_pointer()
    		else:
	    		x = event.x
			y = event.y
			state = event.state

		if not self.dragging:
			return True

		alloc = widget.get_allocation()
		x = (float(x)/alloc.width-0.5)*5
		y = (float(y)/alloc.height-0.5)*-5

    		if state & gtk.gdk.BUTTON1_MASK:
			if self.draggingReference:
				cmd = "<request><reference><position x='%f' y='%f'/></reference></request>\0" % (x,y)
				ss.push(cmd)
				return True

	    		for i in self.selected_sources:
				cmd = "<request><source id='%d'><position x='%f' y='%f'/></source></request>\0" % (i,x,y)
				ss.push(cmd)

    		return True


	def drawLoudspeakers( self, cr ):
		cr.save()
		cr.translate( self.scene.reference.x, self.scene.reference.y )
		cr.rotate( self.scene.reference.azimuth*math.pi/180.0 )
		for i in self.scene.speakers:
			self.drawSpeaker( cr, i )

		cr.set_line_width( 0.02 )
		cr.move_to( 0.2, 0.0 )
		cr.line_to( 0, 0.1 )
		cr.line_to( -0.1, 0 )
		cr.line_to( 0, -0.1 )
		cr.close_path()
		cr.set_source_rgba( 0.2, 0.8, 0.3, 0.7 )
		cr.fill_preserve()
		cr.set_source_rgba( 0.4, 0.8, 0.6, 0.9 )
		cr.stroke()

		cr.restore()


	def drawSource( self, cr, source ):
		cr.save()
		cr.translate( source.x, source.y )

		cr.set_line_width( 0.01 )
		cr.arc( 0, 0, 0.1, 0, 2*math.pi )
		cr.set_source_rgba( 1, 0, 0, 0.6 )
		cr.fill_preserve()
		cr.set_source_rgba( 0.7, 0.7, 0.7, 0.8 )
		cr.stroke()

		if source.type == "plane":
			cr.save()
			cr.rotate( (source.azimuth-90)/180.0*math.pi )
			cr.move_to( -0.3, 0 )
			cr.line_to( 0.3, 0 )
			cr.move_to( 0,0 )
			cr.line_to( 0,0.3 )
			cr.line_to( 0.05, 0.25 )
			cr.set_source_rgba( 0, 0.6, 0.8, 0.7 )
			cr.set_line_width( 0.03 )
			cr.stroke()
			cr.restore()


		lv = max( 0.0, min(1.0, (source.level+40)/46) )
		cr.move_to( -0.1, -0.15 )
		cr.line_to( -0.1+lv*0.2, -0.15 )
		cr.line_to( -0.1+lv*0.2, -0.2 )
		cr.line_to( -0.1, -0.2 )
		cr.close_path()
		cr.set_source_rgba( 0.2, 0.9, 0.2, 0.8 )
		cr.fill()

		cr.set_source_rgba( 0.5, 0.5, 0.5, 0.8 )
		cr.set_line_width( 0.01 )
		cr.move_to( -0.1, -0.15 )
		cr.line_to( 0.1, -0.15 )
		cr.line_to( 0.1, -0.2 )
		cr.line_to( -0.1, -0.2 )
		cr.close_path()
		cr.stroke()

		cr.save()
		volx = (source.volume-packetParser.cap)/(-packetParser.cap+12.0)
		cr.translate( -0.1+volx*0.2,-0.205 )
		cr.move_to( 0,0 )
		cr.line_to( 0.02,-0.04 )
		cr.line_to( -0.02,-0.04 )
		cr.close_path()
		cr.set_source_rgba( 0.5, 0.5, 0.9, 0.8 )
		cr.fill()
		cr.restore()

		cr.set_source_rgba( 1, 1, 1, 1 )
		cr.move_to( 0.1, 0.1 )
		cr.scale( 1, -1 )
		cr.set_font_size( 0.12 )
		cr.show_text( source.name )
		cr.fill()

		cr.restore()

	def drawSpeaker( self, cr, speaker ):
		cr.save()
		cr.translate( speaker.x, speaker.y )
		cr.rotate( (speaker.azimuth-90)/180.0*math.pi )
		cr.set_line_width( 0.3 )
		cr.scale( 0.06, 0.06 )
		cr.move_to( -0.3, -0.5 )
		cr.line_to( 0.3, -0.5 )
		cr.line_to( 0.3, 0 )
		cr.line_to( 1, 0.5 )
		cr.line_to( -1, 0.5 )
		cr.line_to( -0.3, 0 )
		cr.close_path()
		cr.fill()
		cr.restore()



	def update( self ):
		gtk.gdk.threads_enter()
		self.queue_draw()
		gtk.gdk.threads_leave()


w = gtk.Window( gtk.WINDOW_TOPLEVEL )
w.set_default_size( 400, 450 )
da = SSRDraw()

# create pp and ss here.
# need to split up dependencies
# and make the scene contain a reference to the ss.

pp = packetParser.PacketParser( da.scene, da.update )
try:
	ss = ssrsocket.AIOThread( HOSTNAME, PORT, pp.parse_packet )

except:
	dia = gtk.Dialog('Error',
			  None,  #the toplevel wgt of your app
		    	  gtk.DIALOG_MODAL,  #binary flags or'ed together
			  (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
	dia.vbox.pack_start( gtk.Label( "\n  Connection Refused  \n" ) )
	dia.show_all()
	dia.run()
	sys.exit(10)

transport_actions = TransportActionGroup( ss )

uimanager = gtk.UIManager()
uimanager.insert_action_group( transport_actions, 0 )

accelgroup = uimanager.get_accel_group()
w.add_accel_group( accelgroup )

uidescription = """
<toolbar name="Toolbar">
	<toolitem action="rewind"/>
	<toolitem action="pause"/>
	<toolitem action="play"/>
</toolbar>
"""
uimanager.add_ui_from_string( uidescription )


vbox = gtk.VBox()
vbox.pack_start( uimanager.get_widget( "/Toolbar" ), False )
vbox.pack_start( da, True )

w.add( vbox )
w.connect( "destroy", gtk.main_quit )
w.show_all()

ss.start()

gtk.main()
gtk.gdk.threads_leave()
ss.stop()
