/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * Server class (implementation).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#include "server.h"
#include <boost/bind.hpp>

Server::Server(Controller &controller, int port)
  : _controller(controller)
  , _io_service()
  , _acceptor(_io_service
      , boost::asio::ip::tcp::endpoint(boost::asio::ip::tcp::v4(), port))
  , _network_thread(0)
{}

Server::~Server()
{
  this->stop();
}

void
Server::start_accept()
{
  Connection::pointer new_connection = Connection::create(_io_service
      , _controller);

  _acceptor.async_accept(new_connection->socket()
      , boost::bind(&Server::handle_accept, this, new_connection
      , boost::asio::placeholders::error));
}

void
Server::handle_accept(Connection::pointer new_connection
    , const boost::system::error_code &error)
{
  if (!error)
  {
    new_connection->start();
    start_accept();
  }
}

void
Server::start()
{
  _network_thread = new boost::thread(boost::bind(&Server::run, this));
}

void
Server::stop()
{
  VERBOSE2("Stopping network thread ...");
  if (_network_thread)
  {
    _io_service.stop();
    _network_thread->join();
  }
  VERBOSE2("Network thread stopped.");
}

void
Server::run()
{
  start_accept();
  _io_service.run();
}

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
