/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * Dumbrenderer (implementation).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#include "dumbrenderer.h"
#include "stringtools.h"
#include <cstring>

DumbRenderer::DumbRenderer(const Scene &scene, Controller& controller
    , const parameter_map &params)
  : NewRenderer(scene, controller, params)
{
  // In the constructor, the outputs must be created.
  // DumbRenderer only create 10 Outputs tagged with id.

  for (int i=0; i<10; i++)
  {
    parameter_map out_params;
    out_params["port_name"] = "out" + stringtools::A2S(i);
    out_params["id"] = stringtools::A2S(i);
    addOutput(out_params);
  }
}

// TODO: These are boilerplate, i could create them
//       with templates in the renderer.
NewRenderer::Input *
DumbRenderer::create_input(const parameter_map &params)
{
  DumbInput* dumbinput = new DumbInput(*this, params);
  return (Input *) dumbinput;
}

NewRenderer::Output *
DumbRenderer::create_output(const parameter_map &params)
{
  DumbOutput* dumboutput = new DumbOutput(*this, params);
  return (Output *) dumboutput;
}

NewRenderer::RenderSource *
DumbRenderer::create_source(Input *input, const parameter_map &params)
{
  DumbSource* dumbsource = new DumbSource(*this, input, params);
  return (RenderSource *) dumbsource;
}

// TODO: I could get rid of the dynamic casts with templates, too.

void
DumbRenderer::render(NewRenderer::RenderSource *source
    , NewRenderer::Output *output)
{
  DumbInput *dumbinput
    = dynamic_cast<DumbRenderer::DumbInput *> (source->get_input());
  DumbOutput *dumboutput = dynamic_cast<DumbRenderer::DumbOutput *> (output);

  if (dumbinput->id == dumboutput->id) {
    const sample_t *src = dumbinput->get_sample_data();
    sample_t *dst = dumboutput->get_sample_destination();

    memcpy(dst, src, this->_nframes * sizeof(sample_t));
  }
}

DumbRenderer::DumbInput::DumbInput(NewRenderer &renderer
    , const parameter_map &params)
  : NewRenderer::Input(renderer, params)
{
  stringtools::S2A(params["id"], id);
}

DumbRenderer::DumbOutput::DumbOutput(NewRenderer &renderer
    , const parameter_map &params)
  : NewRenderer::Output(renderer, params)
{}

DumbRenderer::DumbSource::DumbSource(NewRenderer &renderer, Input *input
    , const parameter_map &params)
  : NewRenderer::RenderSource(renderer, input, params)
{
  stringtools::S2A(params["id"], id);
}

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
