/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * Dumbrenderer (definition).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#ifndef DUMB_RENDERER_H
#define DUMB_RENDERER_H

#include "renderer.h"
#include "parameter_map.h"

class DumbRenderer : public NewRenderer
{
  public:
    DumbRenderer(const Scene &scene, Controller& controller
        , const parameter_map &params);
    ~DumbRenderer() {}

  protected:
    class DumbInput;
    class DumbOutput;
    class DumbSource;

    virtual Input *create_input(const parameter_map &params);
    virtual Output *create_output(const parameter_map &params);
    virtual RenderSource *create_source(Input *input
        , const parameter_map &params);

    virtual void render(RenderSource *source, Output *output);

    //virtual Source *create_source(const parameter_map_t &params);
    //virtual void remove_source(Source *src);
};

class DumbRenderer::DumbInput : public NewRenderer::Input
{
  public:
    DumbInput(NewRenderer &renderer, const parameter_map &params);
    virtual ~DumbInput() {}

    //private:
    int id;
};

class DumbRenderer::DumbOutput : public NewRenderer::Output
{
  public:
    DumbOutput(NewRenderer &renderer, const parameter_map &params);
    virtual ~DumbOutput() {}
};

class DumbRenderer::DumbSource : public NewRenderer::RenderSource
{
  public:
    DumbSource(NewRenderer &renderer, Input *input
        , const parameter_map &params);
};

#endif

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
