/******************************************************************************
 * Copyright (c) 2006-2012 Quality & Usability Lab                            *
 *                         Deutsche Telekom Laboratories, TU Berlin           *
 *                         Ernst-Reuter-Platz 7, 10587 Berlin, Germany        *
 *                                                                            *
 * This file is part of the SoundScape Renderer (SSR).                        *
 *                                                                            *
 * The SSR is free software:  you can redistribute it and/or modify it  under *
 * the terms of the  GNU  General  Public  License  as published by the  Free *
 * Software Foundation, either version 3 of the License,  or (at your option) *
 * any later version.                                                         *
 *                                                                            *
 * The SSR is distributed in the hope that it will be useful, but WITHOUT ANY *
 * WARRANTY;  without even the implied warranty of MERCHANTABILITY or FITNESS *
 * FOR A PARTICULAR PURPOSE.                                                  *
 * See the GNU General Public License for more details.                       *
 *                                                                            *
 * You should  have received a copy  of the GNU General Public License  along *
 * with this program.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                            *
 * The SSR is a tool  for  real-time  spatial audio reproduction  providing a *
 * variety of rendering algorithms.                                           *
 *                                                                            *
 * http://tu-berlin.de/?id=ssr                  SoundScapeRenderer@telekom.de *
 ******************************************************************************/

/** @file
 * %RendererSubscriber (implementation).
 *
 * $LastChangedDate: 2012-02-01 11:43:25 +0100 (Wed, 01 Feb 2012) $
 * $LastChangedRevision: 1659 $
 * $LastChangedBy: geier.matthias $
 **/

#include "rendersubscriber.h"
#include "stringtools.h"
#include "renderer.h"

RenderSubscriber::RenderSubscriber(NewRenderer &renderer)
  : _renderer(renderer)
{}

RenderSubscriber::~RenderSubscriber()
{}

// Subscriber interface

void
RenderSubscriber::set_loudspeakers(const Loudspeaker::container_t& loudspeakers)
{
  (void) loudspeakers;
}

void
RenderSubscriber::new_source(id_t id)
{
  (void) id;
}

void
RenderSubscriber::delete_source(id_t id)
{
  _renderer.remSourceInput(id);
}

void
RenderSubscriber::delete_all_sources()
{
  _renderer.clearAllSourceInput();
}

bool
RenderSubscriber::set_source_position(id_t id, const Position& position)
{
  Source s = _renderer.getSourceCopy(id);
  s.position = position;
  _renderer.pushSourceChange(id, s);
  return true;
}

bool
RenderSubscriber::set_source_position_fixed(id_t id, const bool& fix)
{
  Source s = _renderer.getSourceCopy(id);
  s.fixed_position = fix;
  _renderer.pushSourceChange(id, s);
  return true;
}

bool
RenderSubscriber::set_source_orientation(id_t id,const Orientation& orientation)
{
  (void) id;
  (void) orientation;
  return true;
}

bool
RenderSubscriber::set_source_gain(id_t id, const float& gain)
{
  Source s = _renderer.getSourceCopy(id);
  s.gain = gain;
  _renderer.pushSourceChange(id, s);
  return true;
}

bool
RenderSubscriber::set_source_mute(id_t id, const bool& mute)
{
  Source s = _renderer.getSourceCopy(id);
  s.mute = mute;
  _renderer.pushSourceChange(id, s);
  return true;
}

bool
RenderSubscriber::set_source_name(id_t id, const std::string& name)
{
  (void) id;
  (void) name;
  return true;
}

bool
RenderSubscriber::set_source_brir_file_name(id_t id, const std::string& name)
{
  (void) id;
  (void) name;
  return true;
}

void
RenderSubscriber::set_amplitude_reference_distance(float distance)
{
  (void) distance;
  return;
}

bool
RenderSubscriber::set_source_model(id_t id, const Source::model_t& model)
{
  Source s = _renderer.getSourceCopy(id);
  s.model = model;
  _renderer.pushSourceChange(id, s);
  return true;
}

bool
RenderSubscriber::set_source_port_name(id_t id, const std::string& port_name)
{
  (void) id;
  (void) port_name;
  return true;
}

bool
RenderSubscriber::set_source_file_name(id_t id, const std::string& file_name)
{
  (void) id;
  (void) file_name;
  return 1;
}

bool
RenderSubscriber::set_source_file_channel(id_t id
    , const unsigned int& file_channel)
{
  (void) id;
  (void) file_channel;
  return 1;
}

bool
RenderSubscriber::set_source_file_length(id_t id, const long int& length)
{
  (void) id;
  (void) length;
  return true;
}

void
RenderSubscriber::set_reference_position(const Position& position)
{
  (void) position;
}

void
RenderSubscriber::set_reference_orientation(const Orientation& orientation)
{
  (void) orientation;
}

void
RenderSubscriber::set_master_volume(float volume)
{
  (void) volume;
}

void
RenderSubscriber::clear_all_active_sources()
{}

void
RenderSubscriber::set_active_sources(
    Loudspeaker::container_t::size_type loudspeaker_id,
    const std::set<id_t>& active_source_ids)
{
  (void) loudspeaker_id;
  (void) active_source_ids;
}

void
RenderSubscriber::set_processing_state(ssr::processing_state_t state)
{
  (void) state;
}

//void RenderSubscriber::set_transport_state(JackClient::State state)
void
RenderSubscriber::set_transport_state(
    const std::pair<bool, jack_nframes_t>& state)
{
  (void) state;
}

void
RenderSubscriber::set_cpu_load(float load)
{
  (void) load;
}

void
RenderSubscriber::set_sample_rate(unsigned int sr)
{
  (void)sr;
}

void
RenderSubscriber::set_master_signal_level(float level)
{
  (void) level;
}

bool
RenderSubscriber::set_source_signal_level(const id_t id, const float& level)
{
  (void) id;
  (void) level;
  return true;
}

// Settings for Vim (http://www.vim.org/), please do not remove:
// vim:softtabstop=2:shiftwidth=2:expandtab:textwidth=80:cindent
